#!/usr/bin/perl

#**************************************************************
# DESCRIPTION
# Perl script used to generate graphics and update rrd files
#**************************************************************
#  CREATION :
# Le 25/04/06 : Creation S. Pilette
#**************************************************************

use RRDs;
use DBI;

# define location of rrdtool databases
my $rrd = '/var/lib/rrd';
# define location of images
my $img = '';

my $dbname = '';
my $dbhost = '';
my $dbport = '';
my $login = "";
my $mdp   = "";

&QueryBySensor();
&ProcessData("Events-by-Severity-on-All", "Events by Severity");

sub QueryBySensor
{
# Get the list of sensors looking like ACH1 or CH3
        $dsn   = "dbi:mysql:dbname=$dbname;host=$dbhost;port$dbport";
        $dbh = DBI->connect($dsn, $login, $mdp) or die "Echec de la connexion\n";

	$requete = "SELECT hostname,sid FROM sensor;";
        $sth = $dbh->prepare($requete);
        $sth->execute();
        while (@enr = $sth -> fetchrow_array) {
		# For each sensor, get data
		&ProcessData("Events-by-Severity-on-$enr[0]", "Events by Severity on $enr[0]" , $enr[1]);
        }
        $sth -> finish;
        $dbh -> disconnect;
}


sub QueryData
{
        $dsn3   = "dbi:mysql:dbname=$dbname;host=$dbhost;port$dbport";
        $dbh3 = DBI->connect($dsn3, $login, $mdp) or die "Echec de la connexion\n";

        $requete3 = "SELECT sig_priority, COUNT(cid) FROM acid_event WHERE sid= $sensor AND sig_priority = $priority GROUP BY sig_priority ORDER BY 1 ASC";
        $sth3 = $dbh3->prepare($requete3);
        $sth3->execute();
        while (@enr3 = $sth3 -> fetchrow_array) {
                $result3 = $enr3[1];
        }
        $sth3 -> finish;
        $dbh3 -> disconnect;
        return $result3 ;
}

sub QueryAllData
{
        $dsn2   = "dbi:mysql:dbname=$dbname;host=$dbhost;port$dbport";
        $dbh2 = DBI->connect($dsn2, $login, $mdp) or die "Echec de la connexion\n";

        $requete2 = "SELECT sig_priority, COUNT(cid) FROM acid_event WHERE sig_priority = $priority GROUP BY sig_priority ORDER BY 1 ASC";
        $sth2 = $dbh2->prepare($requete2);
        $sth2->execute();
        while (@enr2 = $sth2 -> fetchrow_array) {
                $result2 = $enr2[1];
        }
        $sth2 -> finish;
        $dbh2 -> disconnect;
        return $result2 ;
}


sub ProcessData
{
# Get Data from a given sensor or from every sensor if the third parameter is undefined
# inputs: $_[0]: data source name
#	  $_[1]: DS description 
#	  $_[2]: Sensor ID

	# get data

	if ($_[2]) {
		$low    = QueryData($priority=3, $sensor=$_[2]);
		$medium = QueryData($priority=2, $sensor=$_[2]);
		$high   = QueryData($priority=1, $sensor=$_[2]);
	}
	else {
		$low    = QueryAllData($priority=3);
		$medium = QueryAllData($priority=2);
		$high   = QueryAllData($priority=1);
	}

	# remove eol chars
	chomp($low);
	chomp($medium);
	chomp($high);

	print "$_[0] Nb of events low/medium/high: $low, $medium, $high\n";

	# if rrdtool database doesn't exist, create it
	if (! -e "$rrd/$_[0].rrd")
	{
		print "creating rrd database for $_[0] ...\n";
		RRDs::create "$rrd/$_[0].rrd",
			"-s 300",
			"DS:low:DERIVE:600:0:U",
			"DS:medium:DERIVE:600:0:U",
			"DS:high:DERIVE:600:0:U",
			"RRA:AVERAGE:0.5:1:576",
			"RRA:AVERAGE:0.5:12:168",
			"RRA:AVERAGE:0.5:144:62:",
			"RRA:AVERAGE:0.5:288:366";
	}

	# insert values into rrd
	print "Update values for $_[0] : low/medium/high: $low, $medium, $high\n";
	RRDs::update "$rrd/$_[0].rrd",
		"-t", "low:medium:high",
		"N:$low:$medium:$high";

	# create traffic graphs
	&CreateGraph($_[0], "day", $_[1]);
	&CreateGraph($_[0], "week", $_[1]);
	&CreateGraph($_[0], "month", $_[1]); 
	&CreateGraph($_[0], "year", $_[1]);
}

sub CreateGraph
{
# creates graph
# inputs: $_[0]: name
#	  $_[1]: interval (ie, day, week, month, year)
#	  $_[2]: interface description 

	RRDs::graph "$img/$_[0]-$_[1].png",
		"-s -1$_[1]",
		"-t $_[2]",
		"--lazy",
		"-h", "80", "-w", "600",
		"-l 0",
		"-a", "PNG",
		"-v events by severity",
		"DEF:low=$rrd/$_[0].rrd:low:AVERAGE",
		"DEF:medium=$rrd/$_[0].rrd:medium:AVERAGE",
		"DEF:high=$rrd/$_[0].rrd:high:AVERAGE",
		"AREA:low#32CD32:Low",
		"AREA:medium#FFCC00:Medium",
		"AREA:high#990000:High",
		"GPRINT:low:MAX:  Max\\: %5.1lf",
		"GPRINT:low:AVERAGE: Avg\\: %5.1lf",
		"GPRINT:low:LAST: Current\\: %5.1lf events detected\\n",
		"GPRINT:medium:MAX:  Max\\: %5.1lf",
		"GPRINT:medium:AVERAGE: Avg\\: %5.1lf",
		"GPRINT:medium:LAST: Current\\: %5.1lf events detected\\n",
		"GPRINT:high:MAX:  Max\\: %5.1lf",
		"GPRINT:high:AVERAGE: Avg\\: %5.1lf",
		"GPRINT:high:LAST: Current\\: %5.1lf events detected\\n";
	if ($ERROR = RRDs::error) { print "$0: unable to generate $_[0] $_[1] traffic graph: $ERROR\n"; }
}

