<?php
//******************************************************************************
// DESCRIPTION
// SQL stuff
//******************************************************************************
//  CREATION :
//  12/04/2006 : C. Trussardi
//******************************************************************************


//******************************************************************************
function getStats($db)
{
	$query="SELECT COUNT(cid) as total_number, MIN(timestamp) as min_timestamp, MAX(timestamp) as max_timestamp FROM acid_event;";
	$result=$db->query($query);
	return ($result[0]);
}//function getTotalAlertNumber


//******************************************************************************
function getSensorArray($page,$db)
{
	switch($page)
	{
		case "home": 
			$query="SELECT hostname as sensor, COUNT(cid) as total_number, MIN(timestamp) as min_timestamp, MAX(timestamp) as max_timestamp  FROM acid_event, sensor WHERE acid_event.sid = sensor.sid GROUP BY hostname;";
		break;
		
		default:
			$query="SELECT DISTINCT(sid) as sid, hostname FROM sensor;";
		break;
	}//switch
	
	$result=$db->query($query);
	return($result);
}//function getSensorArray


//******************************************************************************
function getSensorHostName($sid,$db)
{
	$query="SELECT hostname FROM sensor WHERE sid = $sid;";
	$result=$db->query($query);
	return($result[0]["hostname"]);
}//function getSebsorHostName


//******************************************************************************
function getDateList($type,$db)
{
	switch ($type)
	{
		case "date" :
			$query = "SELECT dayofyear(timestamp) AS dayofyear, year(timestamp) AS year, monthname(timestamp) AS month, dayofmonth(timestamp) AS dayofmonth FROM acid_event GROUP BY 1 ORDER BY 1 DESC" ;
		break;
		
		case "month":
			$query = "SELECT monthname(timestamp) AS monthname, month(timestamp) as month FROM acid_event GROUP BY 1 ORDER BY 1 DESC" ;
		break;
		
		case "year" :
			$query = "SELECT DISTINCT(year(timestamp)) AS year FROM acid_event ORDER BY timestamp DESC" ;
		break;
	}//switch
	
	$result=$db->query($query);
	return($result);
}//function getDateList


//******************************************************************************
function getClasses($db)
{
	$query = "SELECT DISTINCT(sig_class_name), sig_class_id FROM sig_class" ;
	$result=$db->query($query);
	return($result);
}//function getClasses


//******************************************************************************
function getSignatures($type,$db)
{	
	switch($type)
	{
		case "rule" :
			$query = "SELECT DISTINCT(sig_name) FROM acid_event WHERE sig_name NOT LIKE '(%' AND sig_name NOT LIKE '%Spade:'" ;
		break;
		case "preprocessor" :
			$query = "SELECT DISTINCT(sig_name) FROM acid_event WHERE sig_name LIKE '(%'" ;
		break;
		case "spade" :
			$query = "SELECT DISTINCT(sig_name) FROM acid_event WHERE sig_name LIKE 'Spade:%'" ;
		break;
	}//switch
	$result=$db->query($query);
	return($result);
}//function getRuleSignatures
	
	
//******************************************************************************
function getTopResults($sensor,$hour,$week,$mois,$thisdate,$thishour,$thismonth,$limit,$top,$db)
{
	$s_req="";
	$where="";
	$w_req="";
	$w_title="";
	
	if ($sensor != "") 
	{
		$s_req = " AND sid = $sensor ";
		$where = " on ".getSensorHostName($sensor,$db)."";
	}	

	if ($hour) 
	{
		$w_req = " AND dayofyear(timestamp) = dayofyear(curdate()) ";
		$w_title = "today";
	}

	if ($week != "") 
	{
		$w_req = " AND week(timestamp) = week(curdate()) ";
		$w_title = "this week";
	}

	if ($mois != "") 
	{
		$w_req = " AND month(timestamp) = month(curdate()) ";
		$w_title = "this month";
	}	

	if ($thisdate != "") 
	{
		if ($thishour != "") 
		{
			$w_req = " AND dayofyear(timestamp) = $thisdate AND hour(timestamp) = $thishour ";
			$w_title = "".date("M-d-Y", mktime(0,0,0,1,$thisdate,date("Y")))." At ".$thishour."h";
		} else {
			$w_req = " AND dayofyear(timestamp) = $thisdate ";
			$w_title = $thisdate;
		}	
	}

	if ($thismonth != "") {
		$w_req = " AND month(timestamp) = $thismonth ";
		$w_title = $thismonth;
	}

	switch ($top) {
		case quote_smart("msalr"):
			$title = "$limit most seen alerts $w_title $where" ;
			$sql = "SELECT COUNT(acid_event.sig_name) as total, LEFT(acid_event.sig_name, 50) as signature, sig_sid as sid, MIN(timestamp) as min_timestamp, MAX(timestamp) as max_timestamp FROM acid_event, signature WHERE 1=1 AND acid_event.signature = signature.sig_id ". $w_req ." " . $s_req ." GROUP BY 2 ORDER BY Total DESC LIMIT ". $limit ." ";
		break;
		case quote_smart("msdst"):
			$title = "$limit most targeted hosts $w_title $where" ;
			$sql = "SELECT COUNT(cid) as  total, INET_NTOA(ip_dst) as target, sig_name as signature, COUNT(DISTINCT(ip_src)) as sources_number, MIN(timestamp) AS min_timestamp, MAX(timestamp) as max_timestamp  FROM acid_event WHERE 1=1 ". $w_req ." " . $s_req ." GROUP BY 2 ORDER BY 1 DESC LIMIT ". $limit ." ";
		break;
		case quote_smart("mssrc"):
			$title = "$limit most seen sources $w_title $where" ; 
			$sql = "SELECT COUNT(cid) as total, INET_NTOA(ip_src) as source, sig_name  as signature, MIN(timestamp) AS min_timestamp, MAX(timestamp) as max_timestamp  FROM acid_event WHERE 1=1 ". $w_req ." " . $s_req ." GROUP BY 2, sig_name ORDER BY 1 DESC LIMIT ". $limit ." ";
		break;
        default:
			$limit = 5 ;
            $title = "$limit most seen alerts $where" ;
			$sql = "SELECT COUNT(acid_event.sig_name) as total, LEFT(acid_event.sig_name, 50) as signature, sig_sid as sid, MIN(timestamp) as min_timestamp, MAX(timestamp) as max_timestamp FROM acid_event, signature WHERE 1=1 AND acid_event.signature = signature.sig_id ". $w_req ." " . $s_req ." GROUP BY 2 ORDER BY Total DESC LIMIT ". $limit ." ";
	}
	
	$result=$db->query($sql);
	return(array("data"=>$result,"title"=>$title));
}//function getTopResults


//******************************************************************************
function getGraphResults($sensor,$sig_class_id,$sig_name,$pre_sig_name,$spade_sig_name,$fulltextop,$keyword,$ip_address,$direction,$directionop,$port,$wport,$severity,$protocol,$scope,$thisdate,$thishour,$thismonth,$db)
{
	//Init results array
	$data_result = array();
	
	if (!$ip_address)
	{
		$sql  = "SELECT COUNT(cid) as events_number FROM acid_event WHERE 1=1 ";
	} else {
		$sql  = "SELECT COUNT(cid) as events_number FROM acid_event,acid_ip_cache WHERE 1=1 ";
	}//fi
	
	if ($sensor) 
	{
		$sql .= "AND sid = '". $sensor ."' ";
	}//fi

	if ($sig_class_id) 
	{
        $sql .= "AND sig_class_id = '". $sig_class_id ."' ";
	}//fi

	if ($sig_name) 
	{
        $sql .= "AND sig_name = ". $sig_name ." ";
	}//fi

	if ($pre_sig_name) 
	{
        $sql .= "AND sig_name = ". $pre_sig_name ." ";
	}//fi

	if ($spade_sig_name) 
	{
        $sql .= "AND sig_name = ". $spade_sig_name ." ";
	}//fi

	if ($fulltextop) 
	{
        switch ($fulltextop) {
            case quote_smart("begins"):
				$keyword=str_replace("'","",$keyword);
                $sql .= "AND sig_name LIKE '". $keyword ."%' ";
            break;
            case quote_smart("contains"):
				$keyword=str_replace("'","",$keyword);
                $sql .= "AND sig_name LIKE '%".$keyword."%'";
            break;
        }//switch
	}//fi

	if ($ip_address) 
	{
		$ip_address=str_replace("'","",$ip_address);
		
        switch ($direction) {
            case quote_smart("src"):
                if ($directionop == quote_smart("eq")) {
                    $sql .= "AND acid_ip_cache.ipc_fqdn='". $ip_address ."' AND acid_ip_cache.ipc_ip = acid_event.ip_src ";
                } else {
                    $sql .= "AND acid_ip_cache.ipc_fqdn LIKE '". $ip_address ."%' AND acid_ip_cache.ipc_ip = acid_event.ip_src ";
                }//fi
            break;
			case quote_smart("dst"):
				if ($directionop == quote_smart("eq")) {
					$sql .= "AND acid_ip_cache.ipc_fqdn='". $ip_address ."' AND acid_ip_cache.ipc_ip = acid_event.ip_dst ";
				} else {
					$sql .= "AND acid_ip_cache.ipc_fqdn LIKE '". $ip_address ."%' AND acid_ip_cache.ipc_ip = acid_event.ip_dst ";
				}//fi
			break;
			default:
				if ($directionop == quote_smart("eq")) {
					$sql .= "AND acid_ip_cache.ipc_fqdn='". $ip_address ."' AND (acid_ip_cache.ipc_ip = acid_event.ip_dst OR acid_ip_cache.ipc_ip = acid_event.ip_src) ";
				} else {
					$sql .= "AND acid_ip_cache.ipc_fqdn LIKE '". $ip_address ."%' AND (acid_ip_cache.ipc_ip = acid_event.ip_dst OR acid_ip_cache.ipc_ip = acid_event.ip_src) ";
				}//fi
		}//switch
	}//fi

	if ($port) 
	{
		switch ($wport) {
			case quote_smart("src"):
				$sql .= "AND acid_event.layer4_sport='". $port ."' ";
			break;
			case quote_smart("dst"):
				$sql .= "AND acid_event.layer4_dport='". $port ."' ";
			break;
			default:
				$sql .= "AND (acid_event.layer4_sport='". $port ."' OR acid_event.layer4_dport='". $port ."')";
		}//switch
	}//fi		
	

	if ($severity) 
	{
		$sql .= "AND sig_priority = '". $severity ."' ";
	}

	if ($protocol) 
	{
		if ($protocol == quote_smart("other")) {
			$sql .= "AND ip_proto != 6 AND ip_proto != 1 AND ip_proto != 17 ";
		} else {
			$sql .= "AND ip_proto = '". $protocol ."' ";
		}//fi
	}//fi

	//Find data
	if ($thismonth) 
	{
        for ($day = 0; $day <= 31; $day++) 
		{
            $query = $sql . " AND month(timestamp) = ".$thismonth." AND dayofmonth(timestamp) = ".$day."";
			$result=$db->query($query);
			// Feeding the array of data values
			array_push($data_result, $result[0]["events_number"]);			
        }//for
	}//fi 
	
	if ($thisdate) 
	{			
		if  (!$thishour) {
            $scale = "By hour";
            for ($hour = 0; $hour <= 23; $hour++) 
			{
                $query = $sql . " AND dayofyear(timestamp) = ".$thisdate." AND hour(timestamp) = ".$hour."";
                $result=$db->query($query);
                // Feeding the array of data values
                array_push($data_result, $result[0]["events_number"]); 
			}//for
		}
		else 
		{
		    $scale = "By minute";
		    for ($minute = 0; $minute <= 59; $minute++) 
			{
                $query = $sql . " AND dayofyear(timestamp) = $thisdate AND hour(timestamp) = $thishour AND minute(timestamp) = $minute";
               	$result=$db->query($query);
                // Feeding the array of data values
                array_push($data_result, $result[0]["events_number"]); 
            }//for
		}//fi			
	}//fi
	
	if ($scope) 
	{	
		switch($scope)
		{
			case quote_smart("hour"):
				for ($hour = 0; $hour <= 23; $hour++) 
				{
       		 		$query = $sql . " AND dayofyear(timestamp) = dayofyear(curdate()) AND hour(timestamp) = ".$hour."";
        			$result=$db->query($query);
        			// Feeding the array of data values
        			array_push($data_result, $result[0]["events_number"]); 	
				}//for
			break;
			case quote_smart("week"):
				for ($day = 0; $day <= 6; $day++) 
				{
                	$query = $sql . " AND week(timestamp,1) = week(curdate(),1) AND weekday(timestamp) = ".$day."";
					$result=$db->query($query);
        			// Feeding the array of data values
        			array_push($data_result, $result[0]["events_number"]); 	
				}//for
			break;
			case quote_smart("month"):
			$scale = "By day this month";
        	        for ($day = 1; $day <= 31; $day++) 
					{
        	        	$query = $sql . " AND month(timestamp) = month(curdate()) AND dayofmonth(timestamp) = ".$day."";
					$result=$db->query($query);
        			// Feeding the array of data values
        			array_push($data_result, $result[0]["events_number"]); 	
				}//for
			break;
		}//switch
	}//fi
	
	return($data_result);
	
}//function getGraphResults


//******************************************************************************
function getSearchResults($argsArray,$db)
{
	
	$sql  = "SELECT sid, cid, timestamp, sig_name, sig_priority, INET_NTOA(ip_src) as src, INET_NTOA(ip_dst) as dst, layer4_sport, layer4_dport FROM acid_event WHERE 1=1 ";
	
	foreach ($argsArray as $key=>$value)
	{
		$$key=$value;
	}//foreach
	
	if ($sensor) {
		$sql .= "AND sid = '". $sensor ."' ";
	}//fi

	if ($sigclass) {
        $sql .= "AND sig_class_id = '". $sigclass ."' ";
	}//fi

	if ($signature) {
        $sql .= "AND sig_name = ". $signature ." ";
	}//fi

	if ($pre_signature) {
        $sql .= "AND sig_name = ". $pre_signature ." ";
	}//fi

	if ($spade_signature) {
        $sql .= "AND sig_name = ". $spade_signature ." ";
	}

	if ($fulltextop) {
	
		$keyword=str_replace("'","",$keyword);
	
		switch ($fulltextop) { 
			case quote_smart("begins"):
        		$sql .= "AND sig_name LIKE '". $keyword ."%' ";
			break;
			case quote_smart("contains"):
        		$sql .= "AND sig_name LIKE '%". $keyword ."%' ";
			break;
		}//switch
	}//fi

	if ($ipaddress) {
		switch ($direction) {
			case quote_smart("src"):
				if ($directionop == quote_smart("eq")) {
					$sql .= "AND INET_NTOA(ip_src) = '". $ipaddress ."' ";
				} else {
					$sql .= "AND INET_NTOA(ip_src) LIKE '". $ipaddress ."%' ";
				}//fi
			break;
			case quote_smart("dst"):
                if ($directionop == quote_smart("eq")) {
                    $sql .= "AND INET_NTOA(ip_dst) = '". $ipaddress ."' ";
                } else {
                    $sql .= "AND INET_NTOA(ip_dst) LIKE '". $ipaddress ."%' ";
                }//fi
			break;
			default:
                if ($directionop == quote_smart("eq")) {
        			$sql .= "AND INET_NTOA(ip_src) = '". $ipaddress ."' OR INET_NTOA(ip_dst) = '". $ipaddress ."' ";
                } else {
        			$sql .= "AND INET_NTOA(ip_src) LIKE '". $ipaddress ."%' OR INET_NTOA(ip_dst) LIKE '". $ipaddress ."%' ";
                }//fi
		}//switch
	}//fi

	if ($port) {
        switch ($wport) {
            case quote_smart("src"):
                $sql .= "AND layer4_sport = '". $port ."' ";
            break;
            case quote_smart("dst"):
                $sql .= "AND layer4_dport = '". $port ."' ";
            break;
            default:
                $sql .= "AND layer4_dport = '". $port ."' OR layer4_sport = '". $port ."' ";
        }//switch
	}//fi

	if ($day) {
		if ($day < 10) { $day = "0".$day ; }
	}//fi

	if ($month) {
		if ($month < 10) { $month = "0".$month ; }
	}//fi

	$timestamp = "$year-$month-$day";

	if ($hour) { $timestamp .= " ".$hour ; }
	if ($minute)  { $timestamp .= ":". $minute; }

	if ($when) {
		$sql .= "AND timestamp ". $when ."  '". $timestamp. "%'  ";
	}//fi

	if ($severity) {
		$sql .= "AND sig_priority = '". $severity ."' ";
	}//fi

	if ($protocol) {
		if ($protocol == quote_smart("other")) {
			$sql .= "AND ip_proto != 6 AND ip_proto != 1 AND ip_proto != 17 ";
		} else {
			$sql .= "AND ip_proto = '". $protocol ."' ";
		}//fi
	}//fi
	
	switch($order)
	{
		case "'timestamp'":
			$order="timestamp";
		break;
		case "'severity'":
			$order="severity";
		break;
		case "'sig_name'":
			$order="sig_name";
		break;
		case "'INET_NTOA(ip_src)'":
			$order="INET_NTOA(ip_src)";
		break;
		case "'INET_NTOA(ip_dst)'":
			$order="INET_NTOA(ip_dst)";
		break;
	}//switch
	
	switch($way)
	{
		case "'DESC'":
			$way="DESC";
		break;
		case "'ASC'":
			$way="ASC";
		break;
	}//switch
	
	$sql .= " ORDER BY ". $order . " " . $way . " ";
	
	$number=$db->number_rows($sql);
	
	if ($offset){
		$sql .= " LIMIT ".$limit." OFFSET ".$offset.";";
	} else {
		$sql .= " LIMIT ".$limit.";";
	}//fi
		
	$data=$db->query($sql);
	
	$result["data"]=$data;
	$result["number"]=$number;
	
	return($result);
	
}//function getSearchResults


//******************************************************************************
function getEventDetails($alert_cid,$db)
{
	$result=array();
	
	//Sensor
	$query="SELECT sid FROM acid_event WHERE cid=".$alert_cid.";";
	$data=$db->query($query);
	if(count($data)>0){$alert_sid = $data[0]["sid"];}
	
	//event_name
	$query="SELECT sig_name, signature FROM acid_event WHERE cid = ".$alert_cid." AND sid = ".$alert_sid.";";
	$data=$db->query($query);
	if(count($data)>0){
		$result["event_name"]=$data[0]["sig_name"];
		$signature=$data[0]["signature"];
	}

	//event_severity
	$query="SELECT sig_priority FROM acid_event WHERE cid = ".$alert_cid." AND sid = ".$alert_sid.";";
	$data=$db->query($query);
	if(count($data)>0){$result["event_severity"]=$data[0]["sig_priority"];}
	
	//event_class
	$query="SELECT sig_class_name FROM sig_class, acid_event WHERE acid_event.sig_class_id = sig_class.sig_class_id AND cid = ".$alert_cid." AND acid_event.sid = ".$alert_sid."";
	$data=$db->query($query);
	if(count($data)>0){
		$result["sig_class_name"]=$data[0]["sig_class_name"];
	} else {
		$result["sig_class_name"]="";
	}//fi
	
	//Timestamp and sensor name
	$query="SELECT timestamp, hostname FROM acid_event, sensor WHERE acid_event.sid = sensor.sid AND cid = ". $alert_cid ." AND acid_event.sid = ".$alert_sid."";
	$data=$db->query($query);
	if(count($data)>0){
		$result["timestamp"]=$data[0]["timestamp"];
		$result["sensor"]=$data[0]["hostname"];
	}
	
	// How many times was this kind of event already seen before ?
	$query="SELECT COUNT(cid) as events_number, MIN(timestamp) as min_timestamp, MAX(timestamp) as max_timestamp FROM acid_event WHERE signature = ". $signature ." ORDER BY 1 DESC";
	$data=$db->query($query);
	if(count($data)>0){
		$result["events_number"]=$data[0]["events_number"];
		$result["min_timestamp"]=$data[0]["min_timestamp"];
		$result["max_timestamp"]=$data[0]["max_timestamp"];
	}
	
	//Packet Payload
	$query="SELECT data_payload FROM data WHERE cid  = ".  $alert_cid ." AND sid = ". $alert_sid ."";
	$data=$db->query($query);
	if(count($data)>0){
		$result["payload"]=$data[0]["data_payload"];
	}
	
	//Encoding
	$query="SELECT encoding FROM sensor WHERE sid = ".$alert_sid."";
	$data=$db->query($query);
	if(count($data)>0){
		$result["encoding"]=$data[0]["encoding"];
	}
	
	//Source - Dest - ports - proto
	$query = "SELECT INET_NTOA(ip_src) as source, layer4_sport as source_port, INET_NTOA(ip_dst) as destination, layer4_dport as destination_port, name as protocol FROM acid_event, protocols WHERE ip_proto = protocol AND cid =". $alert_cid ." AND sid = ". $alert_sid .";";
	$data=$db->query($query);
	if(count($data)>0){
		$result["source"]=$data[0]["source"];
		$result["source_port"]=$data[0]["source_port"];
		$result["destination"]=$data[0]["destination"];
		$result["destination_port"]=$data[0]["destination_port"];
		$result["protocol"]=$data[0]["protocol"];
	}//fi
	
	return $result;
}//function getEventDetails
?>